##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################

"""Install Zope Studio Mozilla chrome and components
   ID: $Id: install.py,v 1.1 2000/06/21 11:22:15 mj Exp $
"""

__Version__ = '$Revision: 1.1 $'[11:-2]

import sys, os, getopt, shutil

"""Install all resources and components in their correct places.
Source is the path to the root of a Zope Studio Mozilla checkout.
Dest is the full file path to the Mozilla directory.
If uselinks is non-zero, symbolic links will be created instead of copies.
If reverse is non-zero, newer files from the Mozilla tree are copied back."""
def install(source, dest, uselinks=0, reverse=0):
    # test for installability
    if not os.path.isdir(source):
        print "Source directory doesn't exist or isn't accessible!"
        sys.exit()

    if not os.path.isdir(dest):
        print "Destination directory doesn't exist or isn't accessible!"
        sys.exit()

    files = os.listdir(dest)
    if 'components' not in files or 'chrome' not in files:
        print 'Supplied path is not the Mozilla directory!'
        sys.exit()

    if reverse:
        print "Copying back changes."
    else:
        print "Installing Zope Studio into %s" % dest

    if uselinks: print "Using symbolic linking."

    installComponents(os.path.join(source, 'components'),
        os.path.join(dest, 'components'), uselinks, reverse)
    installChrome(os.path.join(source, 'resources'),
        os.path.join(dest, 'chrome'), uselinks, reverse)

"""Install components.
source is components dir (with src and idl subdirectories)
dest is Mozilla components dir"""
def installComponents(source, dest, uselinks=0, reverse=0):
    print "\t... components"
    # For now copying *.js in src will suffice
    srcDir = os.path.join(source, 'src')
    for file in os.listdir(srcDir):
        if file[-3:] != '.js': continue
        copyFile(os.path.join(srcDir, file), os.path.join(dest, file),
            uselinks, reverse)

"""Install chrome
source is the resources dir
dest is Mozilla chrome dir"""
def installChrome(source, dest, uselinks=0, reverse=0):
    print "\t... chrome"
    # create Zope Studio package dir
    packageDir = os.path.join(dest, 'packages', 'zopestudio')
    if not reverse and not os.path.exists(packageDir): 
        print "\t\t... creating Zope Studio package"
        os.makedirs(packageDir, 0755)

    # All the subdirectories now.
    for type in ('content', 'skin', 'locale'):
        typeSrc = os.path.join(source, type)
        typeDest = os.path.join(packageDir, 'zopestudio', type)
        if not reverse and not os.path.exists(typeDest): 
            print "\t\t... creating %s" % type
            os.makedirs(typeDest, 0755)

        for file in os.listdir(typeSrc):
            srcFile = os.path.join(typeSrc, file)
            if not os.path.isfile(srcFile): continue
            copyFile(srcFile, os.path.join(typeDest, file), uselinks, reverse)

    # Copy manifest file. If indeed copied, add chrome trigger file
    if copyFile(os.path.join(source, 'manifest.rdf'), 
        os.path.join(packageDir, 'manifest.rdf'), uselinks, reverse):
        if not reverse: createInstalledTxt(dest)

"""Create installed-chrome.txt in Mozilla chrome directory (dest), or, if it
exists already, append to it.
"""
def createInstalledTxt(dest):
    print "\t\t... creating installed-chrome.txt information"
    fileName = os.path.join(dest, 'installed-chrome.txt')
    if os.path.isfile(fileName): installedTxt = open(fileName, 'a')
    else: installedTxt = open(fileName, 'w')

    for type in ('content', 'skin'):
        installedTxt.write(type)
        installedTxt.write(',install,url,resource:/chrome/packages/zopestudio/')
        installedTxt.write(os.linesep)

    installedTxt.close()

"""Copy source file to dest file, only if soure is newer than dest"""
def copyFile(source, dest, uselinks=0, reverse=0):
    # switch over if reverse
    if reverse:
        if uselinks: return 0
        source, dest = dest, source
        if not os.path.isfile(source):
            print '\t\t File %s not found' % os.path.basename(source)
            return 0

    if os.path.isfile(dest):
        # Check for newer or up-to-date dest files
        if os.stat(source)[8] <= os.stat(dest)[8]:
            print '\t\t Skipping %s' % os.path.basename(source)
            return 0

    print '\t\t %s' % os.path.basename(source)
    if uselinks:
        os.symlink(source, dest)
    else:
        shutil.copy2(source, dest)

    return 1

usage = """%s [-h] [-lr] [-s source] dest

Install the Zope Studio chrome to a Mozilla install, indicated by 'dest'.

Switches are:

  -h         -- Show this help message.
  -s source  -- Sources can be found at 'source'. Defaults to current dir
  -l         -- Use symlinks instead of copying. Only works on Unix.
  -r         -- Reverse; copy changed files from the Mozilla tree back to the
                source directories. Very handy on platforms other than Unix
                where you can't use symlinks, but do want to use CVS control of
                the whole Zope Studio tree. New files in the Mozilla file tree
                will not be copied!
"""

def main(args):
    try: options, path = getopt.getopt(args[1:], 'hlrs:')
    except:
        print usage % args[0]
        sys.exit()

    if len(path) != 1:
       print usage % args[0]
       sys.exit() 

    dest = path[0]
    source = os.getcwd()
    linking = 0
    reverse = 0

    for (switch, value) in options:
        if switch == '-h': 
            print usage % args[0]
            sys.exit()
        elif switch == '-s':
            source = value
        elif switch == '-l':
            linking = 1
        elif switch == '-r':
            reverse = 1

    if reverse: linking = 0
    
    install(source, dest, uselinks=linking, reverse=reverse)

if __name__ == '__main__': main(sys.argv)
# vim:sw=4:sr:sta:et:sts:tw=80
