/*****************************************************************************

  Zope Public License (ZPL) Version 1.0
  -------------------------------------

  Copyright (c) Digital Creations.  All rights reserved.

  This license has been certified as Open Source(tm).

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are
  met:

  1. Redistributions in source code must retain the above copyright
     notice, this list of conditions, and the following disclaimer.

  2. Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions, and the following disclaimer in
     the documentation and/or other materials provided with the
     distribution.

  3. Digital Creations requests that attribution be given to Zope
     in any manner possible. Zope includes a "Powered by Zope"
     button that is installed by default. While it is not a license
     violation to remove this button, it is requested that the
     attribution remain. A significant investment has been put
     into Zope, and this effort will continue if the Zope community
     continues to grow. This is one way to assure that growth.

  4. All advertising materials and documentation mentioning
     features derived from or use of this software must display
     the following acknowledgement:

       "This product includes software developed by Digital Creations
       for use in the Z Object Publishing Environment
       (http://www.zope.org/)."

     In the event that the product being advertised includes an
     intact Zope distribution (with copyright and license included)
     then this clause is waived.

  5. Names associated with Zope or Digital Creations must not be used to
     endorse or promote products derived from this software without
     prior written permission from Digital Creations.

  6. Modified redistributions of any form whatsoever must retain
     the following acknowledgment:

       "This product includes software developed by Digital Creations
       for use in the Z Object Publishing Environment
       (http://www.zope.org/)."

     Intact (re-)distributions of any official Zope release do not
     require an external acknowledgement.

  7. Modifications are encouraged but must be packaged separately as
     patches to official Zope releases.  Distributions that do not
     clearly separate the patches from the original work must be clearly
     labeled as unofficial distributions.  Modifications which do not
     carry the name Zope may be packaged in any form, as long as they
     conform to all of the clauses above.


  Disclaimer

    THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
    EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
    PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
    CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
    USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
    OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
    OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.


  This software consists of contributions made by Digital Creations and
  many individuals on behalf of Digital Creations.  Specific
  attributions are listed in the accompanying credits file.

 ****************************************************************************/

/*
 *  ObjectBrowser Javascrip behaviours
 *  Version: $Revision: 1.3 $
 *
 *  $Id: objectbrowser.js,v 1.3 2000/07/15 13:15:53 mj Exp $
 */

/*
 * Debug methods.
 */

// debug constants
var bDoTreeFlash = true;

// Uncomment the line that starts with 'dump' for
// debug information on the console.
function debug(sMessage) {
    dump('ZS: ' + sMessage + '\n');
}

// Inspect a DOM node.
function dumpAttributes(oNode) {
    debug("Attributes for " + oNode.nodeName);
    debug("type=" + oNode.nodeType);
    debug("value=" + oNode.nodeValue);

    var oAttributes = oNode.attributes
    if (!oAttributes || oAttributes.length == 0)
        debug("No attributes")
    else
        for (var i=0; i < oAttributes.length; i++) {
            var oAttr = oAttributes.item(i)
            debug("Att " + i + ": " + oAttr.name + " = " + oAttr.value)
        }
}

/*
 * Objects
 */

// LoadObserver observes the status of the remote datasources.
var oLoadObserver = {
    OnBeginLoad: function(oDS) {
//        oDS = oDS.QueryInterface(Components.interfaces.nsIRDFDataSource);
//        debug('OnBeginLoad ' + oDS.URI);
    },

    OnInterrupt: function(oDS) {
//        oDS = oDS.QueryInterface(Components.interfaces.nsIRDFDataSource);
//        debug('OnInterrupt ' + oDS.URI);
    },

    OnResume: function(oDS) {
//        oDS = oDS.QueryInterface(Components.interfaces.nsIRDFDataSource);
//        debug('OnResume ' + oDS.URI);
    },

    OnEndLoad: function(oDS) {
//        oDS = oDS.QueryInterface(Components.interfaces.nsIRDFDataSource);
//        debug('OnEndLoad ' + oDS.URI);
    }
}

/*
 * Constants
 */

var sContainerRDF = '/zsContainerRDF'
var sTreeRoot = '/zsTreeRoot'

/*
 * Globals
 */
// RDF Service component
var oRDFS = Components.classes['component://netscape/rdf/rdf-service'].getService();
oRDFS = oRDFS.QueryInterface(Components.interfaces.nsIRDFService);
// Current location
var sLoc = '';
// Edit window ref
var oEditWindow;

/*
 * Functions
 */

// New Zope location. No trailing slash, please.
function setNewLocation(sURI) {
    debug('New location: ' + sURI);
    sLoc = sURI;
    document.getElementById('location-field').value = sLoc;
    newTreeLocation(sURI);
    displayDetails(sURI + sContainerRDF);
}

// Refresh all datasources. Avoid reloading RDF of details view again when
// doing tree.
function refreshAll() {
    debug('Refreshing details view');

    // Get Details view Composite RDF DataSource
    var oDetails = document.getElementById('detailsview');
    var oDetailsCompDS = oDetails.database;
    oDetailsCompDS = oDetailsCompDS.QueryInterface(Components.interfaces.nsIRDFCompositeDataSource);

    // Get current datasource(s), and refresh them.
    var oCurrRDF = oDetailsCompDS.GetDataSources()
    oCurrRDF = oCurrRDF.QueryInterface(Components.interfaces.nsISimpleEnumerator);
    while (oCurrRDF.hasMoreElements()) {
        oDS = oCurrRDF.getNext();
        oDS = oDS.QueryInterface(Components.interfaces.nsIRDFDataSource);
        sURI = oDS.URI
        if (sURI != 'rdf:localstore') {
            var sDetailsURI = oDS.URI;
            oDS = oDS.QueryInterface(Components.interfaces.nsIRDFXMLSink);
//            oDS.AddXMLSinkObserver(oLoadObserver);
            oDS = oDS.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource);
            oDS.Refresh(false); // not blocking.
            debug('Refreshed ' + sURI);
        }
    }

    // Flash details
    redisplayDetailsWhenLoaded(sDetailsURI, 0);

    debug('Refreshing tree view');

    // Get Tree view Composite RDF DataSource
    var oTree = document.getElementById('treeview');
    var oTreeCompDS = oTree.database;
    oTreeCompDS = oTreeCompDS.QueryInterface(Components.interfaces.nsIRDFCompositeDataSource);

    // Get current datasource(s), and refresh them.
    var oCurrRDF = oTreeCompDS.GetDataSources()
    oCurrRDF = oCurrRDF.QueryInterface(Components.interfaces.nsISimpleEnumerator);
    while (oCurrRDF.hasMoreElements()) {
        oDS = oCurrRDF.getNext();
        oDS = oDS.QueryInterface(Components.interfaces.nsIRDFDataSource);
        sURI = oDS.URI
        if (sURI != sDetailsURI && sURI != 'rdf:localstore') {
            oDS = oDS.QueryInterface(Components.interfaces.nsIRDFXMLSink);
//            oDS.AddXMLSinkObserver(oLoadObserver);
            oDS = oDS.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource);
            oDS.Refresh(false); // not blocking.
            debug('Refreshed ' + sURI);
        }
    }

    // Flash tree
    // Doesn't work properly, as my method of
    // resources loaded detection doesn't work when
    // refreshing. Unfortunately, the proper way is broken
    // right now (crash).
    redisplayTreeWhenLoaded(sLoc + sTreeRoot, 0);
    downloadSubBranchesWhenLoaded(sLoc + sContainerRDF, 0);
}

// Clean up tree and load new datasource
function newTreeLocation(sURI, iTry) {
    // Get Tree view Composite RDF DataSource
    var oTree = document.getElementById('treeview');
    var oCompDS = oTree.database;
    oCompDS = oCompDS.QueryInterface(Components.interfaces.nsIRDFCompositeDataSource);

    // Get current datasource(s), and purge them.
    var oCurrRDF = oCompDS.GetDataSources()
    oCurrRDF = oCurrRDF.QueryInterface(Components.interfaces.nsISimpleEnumerator);
    while (oCurrRDF.hasMoreElements()) {
        oDS = oCurrRDF.GetNext();
        oDS = oDS.QueryInterface(Components.interfaces.nsIRDFDataSource);
        if (oDS.URI != 'rdf:localstore') {
            debug('Removing ' + oDS.URI);
            oCompDS.RemoveDataSource(oDS);
        }
    }

    // Add remote RDF Datasource to details view
    oRDF = oRDFS.GetDataSource(sURI + sTreeRoot);
    oRDF = oRDF.QueryInterface(Components.interfaces.nsIRDFXMLSink);
//    oRDF.AddXMLSinkObserver(oLoadObserver);
    oCompDS.AddDataSource(oRDF);

    oRDF = oRDFS.GetDataSource(sURI + sContainerRDF);
    oRDF = oRDF.QueryInterface(Components.interfaces.nsIRDFXMLSink);
//    oRDF.AddXMLSinkObserver(oLoadObserver);
    oCompDS.AddDataSource(oRDF);

    // Makes sure the tree displays.
    // Also, make sure all datasourecs for open and visible
    // branches are loaded.
    redisplayTreeWhenLoaded(sURI + sTreeRoot, 0);
    downloadSubBranchesWhenLoaded(sURI + sContainerRDF, 0);
}

// Load datasource for branch about to open.
function openBranch(oEl) {
    // RDF Resource to add
    var sRDF = oEl.getAttribute('seeAlso')
    if (!sRDF) return;

    debug('Opening ' + sRDF);

    // Get Composite Datasource of tree
    var oCompDS = document.getElementById('treeview').database;
    oCompDS = oCompDS.QueryInterface(Components.interfaces.nsIRDFCompositeDataSource);

    // Get remote RDF Datasource
    var oRDS = oRDFS.GetDataSource(sRDF);
    oRDS = oRDS.QueryInterface(Components.interfaces.nsIRDFXMLSink);
//    oRDS.AddXMLSinkObserver(oLoadObserver);

    // Add to tree
    oCompDS.AddDataSource(oRDS);

    // Check for subbranches that are open.
    downloadSubBranchesWhenLoaded(sRDF, 0);
}

// branch about to close
function closeBranch(oEl) {
    debug('Closing ' + oEl.getAttribute('seeAlso'));
}

// bring selected container (in detail view) into view
// and select (in tree view)
function selectTreeItem(sURI) {
    var oTree = document.getElementById('treeview');
    var oTreeItems = oTree.getElementsByAttribute('seeAlso', sURI);

    if (oTreeItems && oTreeItems.length) {
        oTItem = oTreeItems[0];
        oTree.clearItemSelection();
        oTree.selectItem(oTItem);
    }
    // TODO: open tree branches if necessary.
    // Use openBranch and setTimeout for this.
}

// Poll datasource for resources. If it has any, load
// datasources for any opened subbranches (openen because
// of persistence).
// Correct way should use nsIRDFXMLSink::AddXMLSinkObserver, but this is
// currently broken (crash). MJ 2000-01-20
function downloadSubBranchesWhenLoaded(sURI, iTry) {
    debug('Polling for ' + sURI + ', try ' + iTry);

    // Cancel poll after 25 tries.
    if (iTry++ == 25) return;

    // Grab DataSource
    var oDS = oRDFS.GetDataSource(sURI);
    oDS = oDS.QueryInterface(Components.interfaces.nsIRDFDataSource);

    // Check if any resources. This means the doc has succesfully loaded.
    var oNodes = oDS.GetAllResources();
    oNodes = oNodes.QueryInterface(Components.interfaces.nsISimpleEnumerator);

    if (!oNodes.hasMoreElements())
        // No nodes (yet), try again in a while.
        return setTimeout('downloadSubBranchesWhenLoaded("' + sURI + '", ' + iTry + ')', 500);

    debug('Branch loaded. Checking for opened subbranches.');
    // Now check for sub branches. If they are listed as open,
    // call openBranch on them. This makes sure all visible
    // datasources are loaded.
    oTreeItems = document.getElementById('treeview').getElementsByAttribute('seeAlso', sURI);
    if (oTreeItems.length && oTreeItems[0] && oTreeItems[0].getAttribute('open') == 'true') {
        if (oTreeItems[0].hasChildNodes()) {
            oSubTreeItems = oTreeItems[0].childNodes[1].childNodes;
            for (var i = 0; i < oSubTreeItems.length; i++)
                if (oSubTreeItems[i].getAttribute('open') == 'true')
                    openBranch(oSubTreeItems[i]);
        }
    }
}

// Poll datasource for resources. If it has any, redisplay details
// Makes sure that newly filled details displays properly
// Correct way should use nsIRDFXMLSink::AddXMLSinkObserver, but this is
// currently broken (crash). MJ 2000-01-20
function redisplayDetailsWhenLoaded(sURI, iTry) {
    debug('Polling for ' + sURI + ', try ' + iTry);

    // Cancel poll after 25 tries.
    if (iTry++ == 25) return;

    // Grab DataSource
    var oDS = oRDFS.GetDataSource(sURI);
    oDS = oDS.QueryInterface(Components.interfaces.nsIRDFDataSource);

    // Check if any resources. This means the doc has succesfully loaded.
    var oNodes = oDS.GetAllResources();
    oNodes = oNodes.QueryInterface(Components.interfaces.nsISimpleEnumerator);

    if (!oNodes.hasMoreElements())
        // No nodes (yet), try again in a while.
        return setTimeout('redisplayDetailsWhenLoaded("' + sURI + '", ' + iTry + ')', 500);

    if (bDoTreeFlash) {
        // Nodes found. Flash details.
        debug(sURI + ' loaded. Flashing.');
        // Corresponding item of details
        oDetails = document.getElementById('detailsview');
        oDetails.setAttribute('ref', oDetails.getAttribute('ref'));
    }
}

// Poll datasource for resources. If it has any, redisplay tree
// Makes sure that newly filled tree displays properly
// Correct way should use nsIRDFXMLSink::AddXMLSinkObserver, but this is
// currently broken (crash). MJ 2000-01-20
function redisplayTreeWhenLoaded(sURI, iTry) {
    debug('Polling for ' + sURI + ', try ' + iTry);

    // Cancel poll after 25 tries.
    if (iTry++ == 25) return;

    // Grab DataSource
    var oDS = oRDFS.GetDataSource(sURI);
    oDS = oDS.QueryInterface(Components.interfaces.nsIRDFDataSource);

    // Check if any resources. This means the doc has succesfully loaded.
    var oNodes = oDS.GetAllResources();
    oNodes = oNodes.QueryInterface(Components.interfaces.nsISimpleEnumerator);

    if (!oNodes.hasMoreElements())
        // No nodes (yet), try again in a while.
        return setTimeout('redisplayTreeWhenLoaded("' + sURI + '", ' + iTry + ')', 500);

    if (bDoTreeFlash) {
        // Nodes found. Flash tree.
        debug(sURI + ' loaded. Flashing.');
        // Corresponding item of tree
        oTree = document.getElementById('treeview');
        oTree.setAttribute('ref', oTree.getAttribute('ref'));
    }
}

// New details to display
// sURI is pointer to RDF datasource
function displayDetails(sURI) {
    var sRDF = sURI
    var sRoot = sURI.slice(0, sURI.lastIndexOf('/'));

    debug('Show details of ' + sRoot);

    // Get Details view Composite RDF DataSource
    var oDetails = document.getElementById('detailsview');
    var oCompDS = oDetails.database;
    oCompDS = oCompDS.QueryInterface(Components.interfaces.nsIRDFCompositeDataSource);

    // Get current datasource(s), and purge them.
    var oCurrRDF = oCompDS.GetDataSources()
    oCurrRDF = oCurrRDF.QueryInterface(Components.interfaces.nsISimpleEnumerator);
    while (oCurrRDF.hasMoreElements()) {
        oDS = oCurrRDF.getNext();
        oDS = oDS.QueryInterface(Components.interfaces.nsIRDFDataSource);
        if (oDS.URI != 'rdf:localstore') {
            debug('Removing ' + oDS.URI);
            oCompDS.RemoveDataSource(oDS);
        }
    }

    // Add remote RDF Datasource to details view
    oCompDS.AddDataSource(oRDFS.GetDataSource(sRDF));

    // Set reference to the root
    oDetails.setAttribute('ref', sRoot);
}

// Sort columns in detail view.
function sortDetails(sColName) {
    debug('Sorting ' + sColName);
    // Get tree column we will sort
    var oNode = document.getElementById(sColName);
    // determine column resource to sort on, and current sort status
    var sResource = oNode.getAttribute('resource');
    var bSortActive = oNode.getAttribute('sortActive');
    var sCurrDir = oNode.getAttribute('sortDirection');

    // default sorting direction
    var sDir="ascending";

    // If already sorted, choose next sorting direction.
    // rotate between natural, ascending and descending
    if (bSortActive == "true") {
        if (sCurrDir == "ascending")       sDir = "descending";
        else if (sCurrDir == "descending") sDir = "natural";
        else                               sDir = "ascending";
    }

    var oXULSortService = Components.classes["component://netscape/rdf/xul-sort-service"].getService();
    oXULSortService = oXULSortService.QueryInterface(Components.interfaces.nsIXULSortService);
    oXULSortService.Sort(oNode, sResource, sDir);

    return false;
}

// Open a URL in the edit window (open the window if needed)
function openEditWindow(sURL) {
    // If Edit window openend and still there, use that.
    if (oEditWindow && oEditWindow.content)
        oEditWindow.content.location.href = sURL;
    else
        oEditWindow = window.openDialog('chrome://navigator/content/navigator.xul',
            'zsEdit', 'chrome,dialog=no,location=no,resizable=yes', sURL);
}


/*
 * Event methods
 */
// onLoad of window
function initWindow() {
    debug('Init window');
    sPersistentLoc = document.getElementById('location-box').getAttribute('location');
    if (sPersistentLoc) setNewLocation(sPersistentLoc);
}

// onUnload of window
function destroyWindow() {
    // Get the current window position/size.
    // Store these into the window attributes (for persistence).
    var oWin = document.getElementById('zs-objectbrowser-window');
    oWin.setAttribute('x', window.screenX);
    oWin.setAttribute('y', window.screenY);
    oWin.setAttribute('height', window.outerHeight);
    oWin.setAttribute('width', window.outerWidth);

    // Get tree view box width, store for persistence.
    oTBox = document.getElementById('treeview-box')
    oTBox.setAttribute('width', oTBox.getAttribute('width'));

    // Get location field, store location for persistence.
    oLocBox = document.getElementById('location-box')
    oLocBox.setAttribute('location', sLoc);
}

// Tree view element clicked
function clickTreeEl(oEvent) {
    var oEl = oEvent.target;
    var bIsTwisty = oEl.getAttribute('class') == 'twisty';
    oEl = oEl.parentNode.parentNode
    if (bIsTwisty) oEl = oEl.parentNode;

    // only left click. Let Mozilla handle right and middle
    if (oEvent.button != 1) return;

    if (bIsTwisty || oEvent.clickCount == 2) {
        // Opening and closing tree branches
        if (oEl.getAttribute('open')) openBranch(oEl);
        else closeBranch(oEl);
    } else {
        // Showing items in detail view
        displayDetails(oEl.getAttribute('seeAlso'));
    }

    oEvent.preventBubble();
}

// Detail view element clicked
function clickDetailEl(oEvent) {
    var oEl = oEvent.target.parentNode.parentNode;
    if (oEvent.clickCount == 2) {
        if (oEl.getAttribute('showInTree') == '1') {
            sURI = oEl.getAttribute('seeAlso')
            displayDetails(sURI);
            selectTreeItem(sURI);
        } else {
            openEditWindow(oEl.getAttribute('id') + '/manage');
        }
    }
    oEvent.preventBubble();
}

// Typing in the location field
function locationChanged(oEvent) {
    if (oEvent.which != 13) return;

    sNewURI = oEvent.target.value;
    while (sNewURI.charAt(sNewURI.length - 1) == '/')
        sNewURI = sNewURI.slice(0, -1);
    setNewLocation(sNewURI);
}

// vim:sw=4:sr:sta:et:sts=4:
