/*****************************************************************************

  Zope Public License (ZPL) Version 1.0
  -------------------------------------

  Copyright (c) Digital Creations.  All rights reserved.

  This license has been certified as Open Source(tm).

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are
  met:

  1. Redistributions in source code must retain the above copyright
     notice, this list of conditions, and the following disclaimer.

  2. Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions, and the following disclaimer in
     the documentation and/or other materials provided with the
     distribution.

  3. Digital Creations requests that attribution be given to Zope
     in any manner possible. Zope includes a "Powered by Zope"
     button that is installed by default. While it is not a license
     violation to remove this button, it is requested that the
     attribution remain. A significant investment has been put
     into Zope, and this effort will continue if the Zope community
     continues to grow. This is one way to assure that growth.

  4. All advertising materials and documentation mentioning
     features derived from or use of this software must display
     the following acknowledgement:

       "This product includes software developed by Digital Creations
       for use in the Z Object Publishing Environment
       (http://www.zope.org/)."

     In the event that the product being advertised includes an
     intact Zope distribution (with copyright and license included)
     then this clause is waived.

  5. Names associated with Zope or Digital Creations must not be used to
     endorse or promote products derived from this software without
     prior written permission from Digital Creations.

  6. Modified redistributions of any form whatsoever must retain
     the following acknowledgment:

       "This product includes software developed by Digital Creations
       for use in the Z Object Publishing Environment
       (http://www.zope.org/)."

     Intact (re-)distributions of any official Zope release do not
     require an external acknowledgement.

  7. Modifications are encouraged but must be packaged separately as
     patches to official Zope releases.  Distributions that do not
     clearly separate the patches from the original work must be clearly
     labeled as unofficial distributions.  Modifications which do not
     carry the name Zope may be packaged in any form, as long as they
     conform to all of the clauses above.


  Disclaimer

    THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
    EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
    PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
    CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
    USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
    OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
    OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.


  This software consists of contributions made by Digital Creations and
  many individuals on behalf of Digital Creations.  Specific
  attributions are listed in the accompanying credits file.

 ****************************************************************************/

/*
 *  MiniBrowser Javascrip behaviours
 *  Version: $Revision: 1.9 $
 *
 *  $Id: zsBrowser.js,v 1.9 2000/07/15 19:58:59 mj Exp $
 */

/*
 * Constants
 */

var sContainerRDF = '/zsContainerRDF';
var sTreeRoot = '/zsTreeRoot';

// FilterListManager takes care of the drop down box with metatypes.
// It's a one-instance service. It will add to this list if neccesary, and
// it can clear the list for you. I don't think there is any need for removal
// of individual metatypes just now.
// Limitation: Only one icon per metatype is recognized.
FilterListManager = {
    metaTypes: {},
    _filterList: null,

    _getFilterList: function() {
        if (!this._filterList)
            this._filterList = document
                .getElementById('browser_filter_list').firstChild;
        return this._filterList;
    },
    
    addMetaType: function(sType, sIconURL) {
        if (this.metaTypes[sType] ) return; // got this one already
        debug('New type ' + sType + ', icon ' + sIconURL);

        this.metaTypes[sType] = sIconURL;
        
        var aMetaTypes = [];
        for (var t in this.metaTypes) aMetaTypes[aMetaTypes.length] = t;
        aMetaTypes.sort();
        
        var oList = this._getFilterList();
        // retrieve the first element of the list, which is
        // an empty item (no filter).
        var oItem = oList.firstChild;
        var iCount = 0;
        // Now step through each next sibling. The new metatype will
        // not be found, and inserted in place.
        while (oItem.nextSibling) {
            oItem = oItem.nextSibling;
            if (oItem.getAttribute('value') != aMetaTypes[iCount]) {
                // metatype array and item list out of sync. Add new
                // metatype before this item.
                var oNewItem = oItem.cloneNode(false);
                oNewItem.setAttribute('value', sType);
                oNewItem.setAttribute('src', sIconURL);
                oNewItem.setAttribute('style', 'list-style-image: url("' + 
                    sIconURL + '");');
                oList.insertBefore(oNewItem, oItem);
                return;
            }
            iCount++;
        }

        // add at the end
        var oNewItem = oItem.cloneNode(false);
        oNewItem.setAttribute('value', sType);
        oNewItem.setAttribute('src', sIconURL);
        oNewItem.setAttribute('style', 'list-style-image: url("' + sIconURL +
            '");');
        oList.appendChild(oNewItem);
    },

    clear: function() {
        debug('Clearing filter list');
        this.metaTypes = {};
        var oList = this._getFilterList();
        // retrieve the first element of the list, which is
        // an empty item (no filter).
        var oFirst = oList.firstChild;
        // Delete the rest.
        while (oFirst.nextSibling) {
            var oItem = oFirst.nextSibling;
            oList.removeChild(oItem);
        }
        this.setBrowserFilter('');
    },

    setBrowserFilter: function(sType) {
        var oRule = document.getElementById('browser_filter_rule');
        debug('Setting browser filter to ' + sType);

        if (sType) oRule.setAttribute('dc:Type', sType);
        else oRule.removeAttribute('dc:Type');
        flashTree();
    }
}


/*
 * Globals
 */

var oTree = null;

/*
 * Functions
 */

// Return tree element
function getTree() {
    if (!oTree) oTree = document.getElementById('browser_tree');
    return oTree;
}

// Clean up tree and load new datasource
function newBrowserLocation(sURI) {
    // Get Tree view Composite RDF DataSource
    var oCompDS = getTree().database
        .QueryInterface(Components.interfaces.nsIRDFCompositeDataSource);

    // Get current datasource(s), and purge them.
    var oCurrRDF = oCompDS.GetDataSources()
        .QueryInterface(Components.interfaces.nsISimpleEnumerator);
    while (oCurrRDF.hasMoreElements()) {
        var oDS = oCurrRDF.getNext();
        oDS = oDS.QueryInterface(Components.interfaces.nsIRDFDataSource);
        if (oDS.URI != 'rdf:localstore') {
            debug('Removing ' + oDS.URI);
            oCompDS.RemoveDataSource(oDS);
        }
    }

    // Clear filter list
    FilterListManager.clear();

    // Add remote RDF Datasource to details view
    var oRDF = oRDFS.GetDataSource(sURI + sTreeRoot);
    // Observe if not yet loaded into cache.
    if (!oRDF.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource)
            .loaded) {
        oRDF = oRDF.QueryInterface(Components.interfaces.nsIRDFXMLSink);
        oRDF.addXMLSinkObserver(new RDFLoadObserver([flashTree],
            [addMetaTypesToFilter]));
    }
    oCompDS.AddDataSource(oRDF);

    oRDF = oRDFS.GetDataSource(sURI + sContainerRDF);
    // Observe if not yet loaded into cache.
    if(!oRDF.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource)
            .loaded) {
        oRDF = oRDF.QueryInterface(Components.interfaces.nsIRDFXMLSink);
        oRDF.addXMLSinkObserver(new RDFLoadObserver([flashTree], 
            [checkChildren], [addMetaTypesToFilter]));
    }
    oCompDS.AddDataSource(oRDF);
}

// Load datasource for branch about to open.
function openBranch(oEl) {
    // RDF Resource to add
    var sRDF = oEl.getAttribute('seeAlso')
    if (!sRDF) return;

    debug('Opening ' + sRDF);

    // Get Composite Datasource of tree
    var oCompDS = getTree().database
        .QueryInterface(Components.interfaces.nsIRDFCompositeDataSource);

    // Get remote RDF Datasource
    var oRDS = oRDFS.GetDataSource(sRDF)
        .QueryInterface(Components.interfaces.nsIRDFXMLSink);

    // Observe if not yet loaded into cache.
    if (!oRDS.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource)
            .loaded) {
        oRDS.addXMLSinkObserver(new RDFLoadObserver([checkChildren],
            [addMetaTypesToFilter]));
    }

    // Add to tree
    oCompDS.AddDataSource(oRDS);
}

// branch about to close
function closeBranch(oEl) {
    debug('Closing ' + oEl.getAttribute('seeAlso'));
}

// Add the metatypes found in the given datasource, that are not
// part of the folder structure, to the filter-on-metatypes list
function addMetaTypesToFilter(oDS) {
    debug('checking for metatypes');
    oDS = oDS.QueryInterface(Components.interfaces.nsIRDFDataSource);
    oChildNodes = oDS.GetAllResources()
        .QueryInterface(Components.interfaces.nsISimpleEnumerator);
    // construct a bunch of literals and property names we need
    var pShowInTree = oRDFS
        .GetResource('http://www.zope.org/Resources/Mozilla/RDF#showInTree');
    var pMetaType = oRDFS
        .GetResource('http://purl.org/metadata/dublin_core#Type');
    var pIconURL = oRDFS
        .GetResource('http://home.netscape.com/NC-rdf#Icon');
    var lZero = oRDFS.GetLiteral('0');
        

    while (oChildNodes.hasMoreElements()) {
        var oResource = oChildNodes.getNext();
        if (oDS.HasAssertion(oResource, pShowInTree, lZero, true)) {
            var sType = oDS.GetTarget(oResource, pMetaType, true)
                .QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
            var sIconURL = oDS.GetTarget(oResource, pIconURL, true)
                .QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
            FilterListManager.addMetaType(sType, sIconURL);
        }
    }
}

// Redisplay tree
function flashTree() {
    debug('flashing tree');
    getTree().builder.rebuild();
}

// Load children of datasoures with opened containers.
function checkChildren(oDS) {
    debug('checking children');
    oDS = oDS.QueryInterface(Components.interfaces.nsIRDFDataSource);
    oTreeItems = getTree().getElementsByAttribute('seeAlso', oDS.URI);
    if (oTreeItems.length && oTreeItems[0] && 
            oTreeItems[0].getAttribute('open') == 'true' &&
            oTreeItems[0].hasChildNodes() && 
            oTreeItems[0].childNodes.length > 1) {
        oSubTreeItems = oTreeItems[0].childNodes[1].childNodes;
        for (var i = 0; i < oSubTreeItems.length; i++)
            if (oSubTreeItems[i].getAttribute('open') == 'true')
                openBranch(oSubTreeItems[i]);
    }
}

/*
 * Event methods
 */

// Tree view element clicked
function onClickTreeEl(oEvent) {
    var oEl = oEvent.target;
    var bIsTwisty = oEl.getAttribute('twisty') == 'true';
    oEl = oEl.parentNode.parentNode
    if (bIsTwisty) oEl = oEl.parentNode;

    // only left click. Let Mozilla handle right and middle
    if (oEvent.button != 1) return;

    debug('isTwisty: ' + bIsTwisty + ', count: ' + (oEvent.detail));
    if (bIsTwisty || oEvent.detail == 2) {
        // Opening and closing tree branches
        oEl.toggleOpenState();
        if (oEl.getAttribute('open') == 'true') openBranch(oEl);
        else closeBranch(oEl);
    } else {
        // Open item properties in propertyeditor
        newPropertiesURI(oEl.getAttribute('id'));
    }

    oEvent.preventBubble();
}

// Filter selection changes
function onFilterListChanged() {
    FilterListManager.setBrowserFilter(document
        .getElementById('browser_filter_list').getAttribute('value'));
}

// vim:sw=4:sr:sta:et:sts=4:
