/*****************************************************************************

  Zope Public License (ZPL) Version 1.0
  -------------------------------------

  Copyright (c) Digital Creations.  All rights reserved.

  This license has been certified as Open Source(tm).

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are
  met:

  1. Redistributions in source code must retain the above copyright
     notice, this list of conditions, and the following disclaimer.

  2. Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions, and the following disclaimer in
     the documentation and/or other materials provided with the
     distribution.

  3. Digital Creations requests that attribution be given to Zope
     in any manner possible. Zope includes a "Powered by Zope"
     button that is installed by default. While it is not a license
     violation to remove this button, it is requested that the
     attribution remain. A significant investment has been put
     into Zope, and this effort will continue if the Zope community
     continues to grow. This is one way to assure that growth.

  4. All advertising materials and documentation mentioning
     features derived from or use of this software must display
     the following acknowledgement:

       "This product includes software developed by Digital Creations
       for use in the Z Object Publishing Environment
       (http://www.zope.org/)."

     In the event that the product being advertised includes an
     intact Zope distribution (with copyright and license included)
     then this clause is waived.

  5. Names associated with Zope or Digital Creations must not be used to
     endorse or promote products derived from this software without
     prior written permission from Digital Creations.

  6. Modified redistributions of any form whatsoever must retain
     the following acknowledgment:

       "This product includes software developed by Digital Creations
       for use in the Z Object Publishing Environment
       (http://www.zope.org/)."

     Intact (re-)distributions of any official Zope release do not
     require an external acknowledgement.

  7. Modifications are encouraged but must be packaged separately as
     patches to official Zope releases.  Distributions that do not
     clearly separate the patches from the original work must be clearly
     labeled as unofficial distributions.  Modifications which do not
     carry the name Zope may be packaged in any form, as long as they
     conform to all of the clauses above.


  Disclaimer

    THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
    EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
    PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
    CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
    USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
    OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
    OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.


  This software consists of contributions made by Digital Creations and
  many individuals on behalf of Digital Creations.  Specific
  attributions are listed in the accompanying credits file.

 ****************************************************************************/

/*
 *  ZopeStudio main functionality
 *  Version: $Revision: 1.4 $
 *
 *  $Id: zsMain.js,v 1.4 2000/07/15 13:15:53 mj Exp $
 */

/*
 * Debug methods.
 */

// Uncomment the line that starts with 'dump' for
// debug information on the console.
function debug(sMessage) {
    dump('ZS: ' + sMessage + '\n');
}

// Inspect a DOM node.
function dumpAttributes(oNode) {
    debug("Attributes for " + oNode.nodeName);
    debug("type=" + oNode.nodeType);
    debug("value=" + oNode.nodeValue);

    var oAttributes = oNode.attributes
    if (!oAttributes || oAttributes.length == 0)
        debug("No attributes")
    else
        for (var i=0; i < oAttributes.length; i++) {
            var oAttr = oAttributes.item(i)
            debug("Att " + i + ": " + oAttr.name + " = " + oAttr.value)
        }
}

/*
 * Globals
 */
// RDF Service component
var oRDFS = getXPCService('component://netscape/rdf/rdf-service',
                            'nsIRDFService');
// Current server URL
var sServer = '';

/*
 * Classes
 */

// LoadObserver observes the status of the remote datasources.
// You pass in arrays describing functions to be called. Each
// array consists of a function, and optionally a list of
// arguments to pass in.
// When the remote RDF has been fully retrieved, the functions
// get called with the datasource as the first argument, and
// the optional extra arguments.
// Example: new RDFLoadObserver([doFoo, '1', 'two'], [doBar]).
// On loading will be called: doFoo(datasource, '1', 'two') and
// doBar(datasource).
function RDFLoadObserver() {
    debug('Adding observer (' + arguments.length + ')');
    this.functions = arguments;
}
RDFLoadObserver.prototype = {
    // Not that this bit ever gets's called. Bugzilla 28258
    onError: function(oDS, iStatus, sError) {
        debug('onError: ' + oDS.QueryInterface(Components.
            interfaces.nsIRDFDataSource).URI + ': ' + sError + ' (' +
            iStatus + ')');
    },
        
    onEndLoad: function(oDS) {
        debug('onEndLoad: ' + oDS.QueryInterface(Components.
            interfaces.nsIRDFDataSource).URI);
        // Call async, because at this moment not all resources have actually
        // been added to the datasource. Only the first is in, I think.
        // So, we call setTimeout to a helper method to call the functions.
        setTimeout(this.functionCallHelper, 1, oDS, this.functions);
    },

    // Apply the arguments to their respective functions
    // with the datasource as first argument.
    functionCallHelper: function(oDS, aFunctions) {
        for (var i = 0; i < aFunctions.length; i++) {
            var oFunc = aFunctions[i][0];
            var aArgs = [oDS].concat(aFunctions[i].slice(1));
            oFunc.apply(null, aArgs);
        }
    }
}

/*
 * Functions
 */

// Return a XPCom service component with interface
function getXPCService(sComponent, sInterface) {
    return Components.classes[sComponent]
        .getService(Components.interfaces[sInterface]);
}

// Return a XPCom class instance with interface
function getXPCInstance(sComponent, sInterface) {
    return Components.classes[sComponent]
        .createInstance(Components.interfaces[sInterface]);
}

// New Zope location. No trailing slash, please.
function setServer(sURI) {
    if (sServer == sURI) return; // nothing changed.

    debug('New server: ' + sURI);
    sServer = sURI;

    // set window title
    var zsWindow = document.getElementById('zopestudio-window');
    window.title = zsWindow.getAttribute('titlemodifier') +
        zsWindow.getAttribute('titlemenuseperator') + sServer;

    // Set up minibrowser
    newBrowserLocation(sURI);
}

// vim:sw=4:sr:sta:et:sts=4:
